#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class SendReportJob < Struct.new(:report_id, :user, :type, :verbose)
      include Redborder::Jobs::JobHelper
      def perform
        Rails.logger.error "\nExecuting send report job\n"
        content = ''
        begin
          content += "Preparing to send report #{report_id}\n"
          if type == 'dashboard'
            report = Report.find(report_id)
            content += "Sending report...\n"
            send_report(report, type)
            content += "Report has been successfully sent!\n"
          elsif type == 'incident'
            report = IncidentGroupReport.find(report_id)
            content += "Sending report...\n"
            send_report(report, type)
            content += "Report has been successfully sent!\n"
          end
        rescue => e
          content += "Error: #{e.message}\n"
          e.backtrace.each { |line| content += "#{line}\n" }
          logit "#{e.class}: #{e.message}" if verbose
          raise e
        ensure
          StoredDelayedJob.create(job: 'SendReportJob', user_id: user.id, content: content)
        end
      end

      def send_report(report, type)
        begin
          Rails.logger.error "\nSending report #{report.name} of type #{type}"
          # Check if we have a list of emails
          if report.email_list.nil? || report.email_list.empty?
            raise StandardError, "No email recipients set for #{type} report ID #{report_id}"
          end
          # create an instance of ActionView, so we can use the render method outside of a controller
          av = ActionView::Base.new
          av.view_paths = ActionController::Base.view_paths
          # need these in case your view constructs any links or references any helper methods.
          av.class_eval do
            include Rails.application.routes.url_helpers
            include ApplicationHelper
            include DashboardHelper
            include WidgetsHelper
            include ReportBlocksHelper
            include ManagersHelper
          end

          case type
          when 'dashboard'
            grover = Grover.new(
              av.render(
                template: 'reports/email.pdf.erb',
                locals: { report: report, blocks: report.blocks_data, user: report.user }
              ),
              orientation: report.orientation_name
            )
            generated_pdf = grover.to_pdf
            

            Tempfile.open(['temp', '.pdf']) do |file|
              file.binmode
              file.write(generated_pdf)
              file.rewind
              report.create_report_historical(user.name, file)
            end
            ReportMailer.report(report).deliver_now
            
          when 'incident'
            # grover = Grover.new(
            #   av.render(
            #     template: 'incident_group_report/email.pdf.erb',
            #     locals: { report: report, user: report.user }
            #   ),
            #   orientation: report.orientation_name
            # )
            # generated_pdf = grover.to_pdf
            

            Tempfile.open(['temp', '.pdf']) do |file|
              file.binmode
              file.write(generated_pdf)
              file.rewind
              report.create_report_historical(user.name, file)
            end
            ReportMailer.report(report).deliver_now
          end


          
        rescue => e
          Rails.logger.error e
        end
      end
    end
  end
end